@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 1997-2015 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997-2016 Oracle and/or its affiliates. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
@@ -127,7 +127,7 @@ class Protocol {
 		throw new IOException("Connect failed");
 	    }
 	}
-	if (enableAPOP) {
+	if (enableAPOP && r.data != null) {
 	    int challStart = r.data.indexOf('<');	// start of challenge
 	    int challEnd = r.data.indexOf('>', challStart); // end of challenge
 	    if (challStart != -1 && challEnd != -1)
@@ -346,17 +346,26 @@ class Protocol {
 	    Response r = simpleCommand("QUIT");
 	    ok = r.ok;
 	} finally {
-	    try {
-		socket.close();
-	    } finally {
-		socket = null;
-		input = null;
-		output = null;
-	    }
+	    close();
 	}
 	return ok;
     }
 
+    /**
+     * Close the connection without sending any commands.
+     */
+    void close() {
+	try {
+	    socket.close();
+	} catch (IOException ex) {
+	    // ignore it
+	} finally {
+	    socket = null;
+	    input = null;
+	    output = null;
+	}
+    }
+
     /**
      * Return the total number of messages and mailbox size,
      * using the STAT command.
