@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 1997-2015 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997-2016 Oracle and/or its affiliates. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
@@ -85,6 +85,10 @@ public class InternetAddress extends Address implements Cloneable {
 	PropUtil.getBooleanSystemProperty(
 			    "mail.mime.address.ignorebogusgroupname", true);
 
+    private static final boolean useCanonicalHostName =
+	PropUtil.getBooleanSystemProperty(
+			    "mail.mime.address.usecanonicalhostname", true);
+
     /**
      * Default constructor.
      */
@@ -455,7 +459,7 @@ public class InternetAddress extends Address implements Cloneable {
 	if (addresses == null || addresses.length == 0)
 	    return null;
 
-	StringBuffer sb = new StringBuffer();
+	StringBuilder sb = new StringBuilder();
 
 	for (int i = 0; i < addresses.length; i++) {
 	    if (i != 0) { // need to append comma
@@ -463,9 +467,14 @@ public class InternetAddress extends Address implements Cloneable {
 		used += 2;
 	    }
 
-	    String s = addresses[i].toString();
+	    // prefer not to split a single address across lines so used=0 below
+	    String s = MimeUtility.fold(0, addresses[i].toString());
 	    int len = lengthOfFirstSegment(s); // length till CRLF
 	    if (used + len > 76) { // overflows ...
+		// smash trailing space from ", " above
+		int curlen = sb.length();
+		if (curlen > 0 && sb.charAt(curlen - 1) == ' ')
+		    sb.setLength(curlen - 1);
 		sb.append("\r\n\t"); // .. start new continuation line
 		used = 8; // account for the starting <tab> char
 	    }
@@ -476,7 +485,8 @@ public class InternetAddress extends Address implements Cloneable {
 	return sb.toString();
     }
 
-    /* Return the length of the first segment within this string.
+    /*
+     * Return the length of the first segment within this string.
      * If no segments exist, the length of the whole line is returned.
      */
     private static int lengthOfFirstSegment(String s) {
@@ -567,7 +577,14 @@ public class InternetAddress extends Address implements Cloneable {
 	String host = null;
 	InetAddress me = InetAddress.getLocalHost();
 	if (me != null) {
-	    host = me.getHostName();
+	    // try canonical host name first
+	    if (useCanonicalHostName)
+		host = me.getCanonicalHostName();
+	    if (host == null)
+		host = me.getHostName();
+	    // if we can't get our name, use local address literal
+	    if (host == null)
+		host = me.getHostAddress();
 	    if (host != null && host.length() > 0 && isInetAddressLiteral(host))
 		host = '[' + host + ']';
 	}
@@ -1284,29 +1301,31 @@ public class InternetAddress extends Address implements Cloneable {
 		    throw new AddressException(
 			    "Domain literal end not at end of domain", addr);
 		inliteral = false;
-	    } else if (c <= 040 || c >= 0177)
+	    } else if (c <= 040 || c >= 0177) {
 		throw new AddressException(
 				"Domain contains control or whitespace", addr);
-	    // RFC 2822 rule
-	    //if (specialsNoDot.indexOf(c) >= 0)
-	    /*
-	     * RFC 1034 rule is more strict
-	     * the full rule is:
-	     * 
-	     * <domain> ::= <subdomain> | " "
-	     * <subdomain> ::= <label> | <subdomain> "." <label>
-	     * <label> ::= <letter> [ [ <ldh-str> ] <let-dig> ]
-	     * <ldh-str> ::= <let-dig-hyp> | <let-dig-hyp> <ldh-str>
-	     * <let-dig-hyp> ::= <let-dig> | "-"
-	     * <let-dig> ::= <letter> | <digit>
-	     */
-	    if (!inliteral) {
-		if (!(Character.isLetterOrDigit(c) || c == '-' || c == '.'))
-		    throw new AddressException(
+	    } else {
+		// RFC 2822 rule
+		//if (specialsNoDot.indexOf(c) >= 0)
+		/*
+		 * RFC 1034 rule is more strict
+		 * the full rule is:
+		 * 
+		 * <domain> ::= <subdomain> | " "
+		 * <subdomain> ::= <label> | <subdomain> "." <label>
+		 * <label> ::= <letter> [ [ <ldh-str> ] <let-dig> ]
+		 * <ldh-str> ::= <let-dig-hyp> | <let-dig-hyp> <ldh-str>
+		 * <let-dig-hyp> ::= <let-dig> | "-"
+		 * <let-dig> ::= <letter> | <digit>
+		 */
+		if (!inliteral) {
+		    if (!(Character.isLetterOrDigit(c) || c == '-' || c == '.'))
+			throw new AddressException(
 				    "Domain contains illegal character", addr);
-		if (c == '.' && lastc == '.')
-		    throw new AddressException(
-				    "Domain contains dot-dot", addr);
+		    if (c == '.' && lastc == '.')
+			throw new AddressException(
+					"Domain contains dot-dot", addr);
+		}
 	    }
 	    lastc = c;
 	}
