@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 1997-2015 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997-2016 Oracle and/or its affiliates. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
@@ -393,7 +393,7 @@ public class MimeMessage extends Message implements MimePart {
 	if (address == null)
 	    removeHeader("From");
 	else
-	    setHeader("From", address.toString());
+	    setHeader("From", MimeUtility.fold(6, address.toString()));
     }
 
     /**
@@ -497,7 +497,7 @@ public class MimeMessage extends Message implements MimePart {
 	if (address == null)
 	    removeHeader("Sender");
 	else
-	    setHeader("Sender", address.toString());
+	    setHeader("Sender", MimeUtility.fold(8, address.toString()));
     }
 
     /**
