@@ -1,8 +1,8 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 2013-2015 Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 2013-2015 Jason Mehrens. All rights reserved.
+ * Copyright (c) 2013-2016 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013-2016 Jason Mehrens. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
@@ -60,7 +60,7 @@ import static org.junit.Assert.*;
  * @author Jason Mehrens
  * @since JavaMail 1.5.2
  */
-public class CollectorFormatterTest {
+public class CollectorFormatterTest extends AbstractLogging {
 
     /**
      * See LogManager.
@@ -88,21 +88,6 @@ public class CollectorFormatterTest {
         LogManager.getLogManager().getProperty("");
     }
 
-    @SuppressWarnings("SleepWhileInLoop")
-    private static void tick() throws InterruptedException {
-        final long delay = 1L;
-        long then = System.currentTimeMillis();
-        for (int i = 0; i < Short.MAX_VALUE; i++) {
-            long now = System.currentTimeMillis();
-            long delta = (now - then);
-            if (delta >= delay) {
-                return;
-            }
-            Thread.sleep(delay - delta);
-        }
-        throw new AssertionError(then + " " + System.currentTimeMillis());
-    }
-
     @BeforeClass
     public static void setUpClass() throws Exception {
         checkJVMOptions();
@@ -123,6 +108,12 @@ public class CollectorFormatterTest {
         fullFence();
     }
 
+    @Test
+    public void testDeclaredClasses() throws Exception {
+        Class<?>[] declared = CollectorFormatter.class.getDeclaredClasses();
+        assertEquals(Arrays.toString(declared), 0, declared.length);
+    }
+
     @Test
     public void testFormatHead() {
         String msg = "message";
@@ -159,6 +150,7 @@ public class CollectorFormatterTest {
         private static final long serialVersionUID = 1L;
 
         int inferred;
+        @SuppressWarnings("FieldMayBeFinal")
         private transient Formatter f;
 
         TestFormatterAccept(Level level, CollectorFormatter f) {
@@ -190,11 +182,11 @@ public class CollectorFormatterTest {
                 new SeverityComparator());
         LogRecord first = new LogRecord(Level.INFO, "");
         first.setThrown(new Throwable());
-        first.setMillis(1L);
+        setEpochMilli(first, 1L);
         f.format(first);
 
         TestFormatterAccept r = new TestFormatterAccept(Level.FINE, f);
-        r.setMillis(2L);
+        setEpochMilli(r, 2L);
         r.setThrown(new Throwable());
         f.format(r);
         assertEquals(1, r.inferred);
@@ -206,6 +198,7 @@ public class CollectorFormatterTest {
 
         private static final long serialVersionUID = 1L;
         int inferred;
+        @SuppressWarnings("FieldMayBeFinal")
         private transient Formatter f;
 
         public TestFormatAcceptAndUpdate(Level level, CollectorFormatter f) {
@@ -217,7 +210,7 @@ public class CollectorFormatterTest {
         public String getSourceMethodName() {
             if (++inferred == 1) {
                 LogRecord r = new LogRecord(Level.INFO, "");
-                r.setMillis(1L);
+                setEpochMilli(r, 1L);
                 f.format(r);
             }
             return super.getSourceMethodName();
@@ -233,7 +226,7 @@ public class CollectorFormatterTest {
 
         TestFormatAcceptAndUpdate r
                 = new TestFormatAcceptAndUpdate(Level.SEVERE, f);
-        r.setMillis(2L);
+        setEpochMilli(r, 2L);
         r.setThrown(new Throwable());
         f.format(r);
         assertEquals(2, r.inferred);
@@ -446,11 +439,11 @@ public class CollectorFormatterTest {
                 (Formatter) null,
                 (Comparator<LogRecord>) null);
 
-        tick(); //Make sure the max not equal to the start time.
+        tickMilli(); //Make sure the max not equal to the start time.
 
         final String min = minF.getTail((Handler) null);
         NumberFormat.getIntegerInstance().parse(min);
-        tick();
+        tickMilli();
 
         //Next min is not old min.
         String next = minF.getTail((Handler) null);
@@ -475,23 +468,23 @@ public class CollectorFormatterTest {
         long min = 100L;
 
         LogRecord r = new LogRecord(Level.SEVERE, msg);
-        r.setMillis(min + 1000L);
+        setEpochMilli(r, min + 1000L);
         f.format(r);
 
         r = new LogRecord(Level.SEVERE, msg);
-        r.setMillis(min + 2000L);
+        setEpochMilli(r, min + 2000L);
         f.format(r);
 
         r = new LogRecord(Level.SEVERE, msg);
-        r.setMillis(min);
+        setEpochMilli(r, min);
         f.format(r);
 
         r = new LogRecord(Level.SEVERE, msg);
-        r.setMillis(min + 3000L);
+        setEpochMilli(r, min + 3000L);
         f.format(r);
 
         r = new LogRecord(Level.SEVERE, msg);
-        r.setMillis(min + 4000L);
+        setEpochMilli(r, min + 4000L);
         f.format(r);
 
         String result = f.getTail((Handler) null);
@@ -506,7 +499,7 @@ public class CollectorFormatterTest {
         String now = f.getTail((Handler) null);
         Number num = NumberFormat.getIntegerInstance().parse(now);
         assertFalse(Long.MIN_VALUE == num.longValue());
-        tick();
+        tickMilli();
         String next = f.getTail((Handler) null);
         assertFalse(NumberFormat.getIntegerInstance().parse(now).longValue()
                 == Long.MIN_VALUE);
@@ -524,23 +517,23 @@ public class CollectorFormatterTest {
         long high = 4000L;
 
         LogRecord r = new LogRecord(Level.SEVERE, msg);
-        r.setMillis(min + 1000L);
+        setEpochMilli(r, min + 1000L);
         f.format(r);
 
         r = new LogRecord(Level.SEVERE, msg);
-        r.setMillis(min + high);
+        setEpochMilli(r, min + high);
         f.format(r);
 
         r = new LogRecord(Level.SEVERE, msg);
-        r.setMillis(min + 2000L);
+        setEpochMilli(r, min + 2000L);
         f.format(r);
 
         r = new LogRecord(Level.SEVERE, msg);
-        r.setMillis(min);
+        setEpochMilli(r, min);
         f.format(r);
 
         r = new LogRecord(Level.SEVERE, msg);
-        r.setMillis(min + 3000L);
+        setEpochMilli(r, min + 3000L);
         f.format(r);
 
         String result = f.getTail((Handler) null);
@@ -554,11 +547,11 @@ public class CollectorFormatterTest {
                 (Comparator<LogRecord>) null);
 
         LogRecord r = new LogRecord(Level.SEVERE, "");
-        r.setMillis(100);
+        setEpochMilli(r, 100);
         f.format(r);
 
         r = new LogRecord(Level.SEVERE, "");
-        r.setMillis(200);
+        setEpochMilli(r, 200);
         f.format(r);
 
         //Check that the min and max are different.
@@ -568,7 +561,7 @@ public class CollectorFormatterTest {
                 (output.length() - fence) - 1));
 
         r = new LogRecord(Level.SEVERE, "");
-        r.setMillis(400);
+        setEpochMilli(r, 400);
         f.format(r);
 
         //Previous max is 200 so at this point the min and max better be 400.
@@ -617,18 +610,18 @@ public class CollectorFormatterTest {
                 + "{8,time,EEE, MMM dd HH:mm:ss:S ZZZ yyyy}\n";
         CollectorFormatter cf = new CollectorFormatter(p);
         LogRecord min = new LogRecord(Level.SEVERE, "");
-        min.setMillis(1248203502449L);
+        setEpochMilli(min, 1248203502449L);
         cf.format(min);
 
         int count = 290;
         for (int i = 0; i < count; ++i) {
             LogRecord mid = new LogRecord(Level.SEVERE, "");
-            mid.setMillis(min.getMillis());
+            setEpochMilli(mid, min.getMillis());
             cf.format(mid);
         }
 
         LogRecord max = new LogRecord(Level.SEVERE, "");
-        max.setMillis(1258723764000L);
+        setEpochMilli(max, 1258723764000L);
         cf.format(max);
         Object[] args = new Object[9];
         args[3] = count + 2L;
@@ -646,17 +639,17 @@ public class CollectorFormatterTest {
                 + "|86400000<{7,date} and {8,date}}\n";
         CollectorFormatter cf = new CollectorFormatter(p);
         LogRecord min = new LogRecord(Level.SEVERE, "");
-        min.setMillis(1248203502449L);
+        setEpochMilli(min, 1248203502449L);
         cf.format(min);
 
         for (int i = 0; i < 71; ++i) {
             LogRecord mid = new LogRecord(Level.SEVERE, "");
-            mid.setMillis(min.getMillis());
+            setEpochMilli(mid, min.getMillis());
             cf.format(mid);
         }
 
         LogRecord max = new LogRecord(Level.SEVERE, "");
-        max.setMillis(min.getMillis() + 110500);
+        setEpochMilli(max, min.getMillis() + 110500);
         cf.format(max);
 
         String output = cf.getTail((Handler) null);
@@ -665,11 +658,11 @@ public class CollectorFormatterTest {
         cf.format(min);
         for (int i = 0; i < 114; ++i) {
             LogRecord mid = new LogRecord(Level.SEVERE, "");
-            mid.setMillis(min.getMillis());
+            setEpochMilli(mid, min.getMillis());
             cf.format(mid);
         }
 
-        max.setMillis(min.getMillis() + 2591000000L);
+        setEpochMilli(max, min.getMillis() + 2591000000L);
         cf.format(max);
 
         output = cf.getTail((Handler) null);
@@ -682,17 +675,17 @@ public class CollectorFormatterTest {
                 + "|86400000<{7,date} and {8,date}}\n";
         CollectorFormatter cf = new CollectorFormatter(p);
         LogRecord min = new LogRecord(Level.SEVERE, "");
-        min.setMillis(1248203502449L);
+        setEpochMilli(min, 1248203502449L);
 
         cf.format(min);
         for (int i = 0; i < 114; ++i) {
             LogRecord mid = new LogRecord(Level.SEVERE, "");
-            mid.setMillis(min.getMillis());
+            setEpochMilli(mid, min.getMillis());
             cf.format(mid);
         }
 
         LogRecord max = new LogRecord(Level.SEVERE, "");
-        max.setMillis(min.getMillis() + 2591000000L);
+        setEpochMilli(max, min.getMillis() + 2591000000L);
         cf.format(max);
 
         String output = cf.getTail((Handler) null);
@@ -863,6 +856,25 @@ public class CollectorFormatterTest {
         }
     }
 
+    @Test
+    public void testFormat() throws Exception {
+        final String p = CollectorFormatter.class.getName();
+        Properties props = new Properties();
+        final String expect = CollectorFormatterTest.class.getName();
+        props.put(p.concat(".format"), expect);
+        LogManager manager = LogManager.getLogManager();
+        try {
+            read(manager, props);
+            CollectorFormatter cf = new CollectorFormatter();
+            LogRecord first = new LogRecord(Level.SEVERE, Level.SEVERE.getName());
+            assertEquals("", cf.format(first));
+            String result = cf.getTail((Handler) null);
+            assertEquals(expect, result);
+        } finally {
+            manager.reset();
+        }
+    }
+
     @Test
     public void testFormatter() throws Exception {
         final String p = CollectorFormatter.class.getName();
@@ -889,11 +901,11 @@ public class CollectorFormatterTest {
         CollectorFormatter f = new CollectorFormatter("{9}", (Formatter) null,
                 (Comparator<LogRecord>) null);
         LogRecord r = new LogRecord(Level.SEVERE, "");
-        r.setMillis(25L);
+        setEpochMilli(r, 25L);
         f.format(r);
 
         r = new LogRecord(Level.SEVERE, "");
-        r.setMillis(100L);
+        setEpochMilli(r, 100L);
         f.format(r);
 
         String init = f.getTail((Handler) null);
@@ -908,7 +920,7 @@ public class CollectorFormatterTest {
 
         String init = f.getTail((Handler) null);
         NumberFormat.getIntegerInstance().parse(init);
-        tick();
+        tickMilli();
 
         assertTrue(init.equals(f.getTail((Handler) null)));
     }
@@ -923,7 +935,7 @@ public class CollectorFormatterTest {
         String init = f.getTail((Handler) null);
         DateFormat df = new SimpleDateFormat(DATE_TIME_FMT);
         Date dt = df.parse(init);
-        tick();
+        tickMilli();
 
         assertTrue(init.equals(f.getTail((Handler) null)));
         assertTrue(dt.equals(df.parse(f.getTail((Handler) null))));
@@ -936,7 +948,7 @@ public class CollectorFormatterTest {
 
         String now = f.getTail((Handler) null);
         NumberFormat.getIntegerInstance().parse(now);
-        tick();
+        tickMilli();
 
         assertFalse(now.equals(f.getTail((Handler) null)));
     }
@@ -951,7 +963,7 @@ public class CollectorFormatterTest {
         String init = f.getTail((Handler) null);
         DateFormat df = new SimpleDateFormat(DATE_TIME_FMT);
         Date dt = df.parse(init);
-        tick();
+        tickMilli();
 
         assertFalse(init.equals(f.getTail((Handler) null)));
         assertFalse(dt.equals(df.parse(f.getTail((Handler) null))));
@@ -964,7 +976,7 @@ public class CollectorFormatterTest {
 
         String up = f.getTail((Handler) null);
         NumberFormat.getIntegerInstance().parse(up);
-        tick();
+        tickMilli();
 
         assertFalse(up.equals(f.getTail((Handler) null)));
     }
@@ -1038,10 +1050,19 @@ public class CollectorFormatterTest {
         }
     }
 
-    private void read(LogManager manager, Properties props) throws IOException {
-        ByteArrayOutputStream out = new ByteArrayOutputStream(512);
-        props.store(out, CollectorFormatterTest.class.getName());
-        manager.readConfiguration(new ByteArrayInputStream(out.toByteArray()));
+    @Test
+    public void testJavaMailLinkage() throws Exception {
+        testJavaMailLinkage(CollectorFormatter.class);
+    }
+
+    @Test
+    public void testLogManagerModifiers() throws Exception {
+        testLogManagerModifiers(CollectorFormatter.class);
+    }
+
+    @Test
+    public void testWebappClassLoaderFieldNames() throws Exception {
+        testWebappClassLoaderFieldNames(CollectorFormatter.class);
     }
 
     /**
