@@ -1,8 +1,8 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 2013-2015 Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 2013-2015 Jason Mehrens. All rights reserved.
+ * Copyright (c) 2013-2016 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013-2016 Jason Mehrens. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
@@ -41,6 +41,7 @@
 package com.sun.mail.util.logging;
 
 import java.io.*;
+import java.lang.reflect.Method;
 import java.net.SocketException;
 import java.util.*;
 import java.util.logging.Level;
@@ -57,7 +58,7 @@ import static org.junit.Assert.*;
  * @author Jason Mehrens
  * @since JavaMail 1.5.2
  */
-public class CompactFormatterTest {
+public class CompactFormatterTest extends AbstractLogging {
 
     private static final String UNKNOWN_CLASS_NAME
             = CompactFormatterTest.class.getName().concat("Foo");
@@ -119,6 +120,29 @@ public class CompactFormatterTest {
         fullFence();
     }
 
+    @Test
+    public void testDeclaredClasses() throws Exception {
+        testLoadDeclaredClasses(CompactFormatter.class);
+    }
+
+    @Test
+    public void testFormat() throws Exception {
+        final String p = CompactFormatter.class.getName();
+        Properties props = new Properties();
+        props.put(p.concat(".format"), "%9$s");
+        LogManager manager = LogManager.getLogManager();
+        try {
+            read(manager, props);
+            CompactFormatter cf = new CompactFormatter();
+            LogRecord first = new LogRecord(Level.SEVERE, Level.INFO.getName());
+            first.setSequenceNumber(Short.MAX_VALUE);
+            String result = cf.format(first);
+            assertEquals(String.valueOf((int) Short.MAX_VALUE), result);
+        } finally {
+            manager.reset();
+        }
+    }
+
     @Test
     public void testNewFormatterWithPattern() {
         CompactFormatter cf = new CompactFormatter("%4$s");
@@ -452,6 +476,23 @@ public class CompactFormatterTest {
                 cf.format(r));
     }
 
+    @Test
+    public void testFormatZoneDateTime() throws Exception {
+        LogRecord r = new LogRecord(Level.SEVERE, "");
+        Object zdt = LogManagerProperties.getZonedDateTime(r);
+        if (zdt != null) {
+            String p = "%1$tb %1$td, %1$tY %1$tl:%1$tM:%1$tS.%1$tN %1$Tp";
+            CompactFormatter cf = new CompactFormatter(p);
+            assertEquals(String.format(p, zdt), cf.format(r));
+        } else {
+            try {
+                Method m = LogRecord.class.getMethod("getInstant");
+                fail(m.toString());
+            } catch (final NoSuchMethodException expect) {
+            }
+        }
+    }
+
     @Test(expected=NullPointerException.class)
     public void testFormatNull() {
         CompactFormatter cf = new CompactFormatter();
@@ -723,7 +764,7 @@ public class CompactFormatterTest {
         LogRecord r = new LogRecord(Level.SEVERE, "Encoding failed.");
         r.setSourceClassName("MyClass");
         r.setSourceMethodName("fatal");
-        r.setMillis(1258723764000L);
+        setEpochMilli(r, 1258723764000L);
         RuntimeException npe = new NullPointerException();
         StackTraceElement frame = new StackTraceElement("java.lang.String",
                 "getBytes", "String.java", 913);
@@ -740,7 +781,7 @@ public class CompactFormatterTest {
         LogRecord r = new LogRecord(Level.SEVERE, "Unable to send notification.");
         r.setSourceClassName("MyClass");
         r.setSourceMethodName("fatal");
-        r.setMillis(1258723764000L);
+        setEpochMilli(r, 1258723764000L);
 
         Exception t = new SocketException("Permission denied: connect");
         t = new MessagingException("Couldn't connect to host", t);
@@ -758,7 +799,7 @@ public class CompactFormatterTest {
         r.setThreadID(38);
         r.setSourceClassName("MyClass");
         r.setSourceMethodName("fatal");
-        r.setMillis(1248203502449L);
+        setEpochMilli(r, 1248203502449L);
 
         Exception t = new SocketException("Permission denied: connect");
 
@@ -1051,6 +1092,21 @@ public class CompactFormatterTest {
         assertNull(cf.toAlternate((String) null));
     }
 
+    @Test
+    public void testJavaMailLinkage() throws Exception {
+        testJavaMailLinkage(CompactFormatter.class);
+    }
+
+    @Test
+    public void testLogManagerModifiers() throws Exception {
+        testLogManagerModifiers(CompactFormatter.class);
+    }
+
+    @Test
+    public void testWebappClassLoaderFieldNames() throws Exception {
+        testWebappClassLoaderFieldNames(CompactFormatter.class);
+    }
+
     private static String rpad(String s, int len, String p) {
         if (s.length() < len) {
             StringBuilder sb = new StringBuilder(len);
