@@ -1,8 +1,8 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 2015 Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 2015 Jason Mehrens. All rights reserved.
+ * Copyright (c) 2015-2016 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015-2016 Jason Mehrens. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
@@ -40,8 +40,8 @@
  */
 package com.sun.mail.util.logging;
 
-import java.io.*;
 import java.lang.reflect.Field;
+import java.util.Arrays;
 import java.util.Properties;
 import java.util.logging.*;
 import org.junit.*;
@@ -52,7 +52,7 @@ import static org.junit.Assert.*;
  *
  * @author Jason Mehrens
  */
-public class DurationFilterTest {
+public class DurationFilterTest extends AbstractLogging {
 
     public DurationFilterTest() {
     }
@@ -75,6 +75,12 @@ public class DurationFilterTest {
         assertEquals(LogManager.class, LogManager.getLogManager().getClass());
     }
 
+    @Test
+    public void testDeclaredClasses() throws Exception {
+        Class<?>[] declared = DurationFilter.class.getDeclaredClasses();
+        assertEquals(Arrays.toString(declared), 0, declared.length);
+    }
+
     @Test
     public void testClone() throws Exception {
         DurationFilterExt source = new DurationFilterExt();
@@ -102,13 +108,13 @@ public class DurationFilterTest {
 
         //Allow
         for (int i = 0; i < records; i++) {
-            r.setMillis(millis);
+            setEpochMilli(r, millis);
             assertTrue(Integer.toString(i), sf.isLoggable(r));
         }
 
         Filter clone = sf.clone();
         for (int i = 0; i < records; i++) {
-            r.setMillis(millis);
+            setEpochMilli(r, millis);
             String m = Integer.toString(i);
             assertFalse(m, sf.isLoggable(r));
             assertTrue(m, clone.isLoggable(r));
@@ -119,7 +125,6 @@ public class DurationFilterTest {
     }
 
     @Test(timeout = 15000)
-    @SuppressWarnings("SleepWhileInLoop")
     public void testIsLoggableNow() throws Exception {
         final int records = 10;
         final int duration = 1000;
@@ -142,11 +147,7 @@ public class DurationFilterTest {
         assertFalse(sf.isLoggable());
         assertFalse(sf.isLoggable(r));
 
-        //Cool down and allow.
-        final long then = System.currentTimeMillis();
-        do {
-            Thread.sleep(duration + 100);
-        } while ((System.currentTimeMillis() - then) < duration);
+        tickMilli(duration + 100); //Cool down and allow.
 
         for (int i = 0; i < records; i++) {
             r = new LogRecord(lvl, "");
@@ -160,7 +161,6 @@ public class DurationFilterTest {
     }
 
     @Test(timeout = 15000)
-    @SuppressWarnings("SleepWhileInLoop")
     public void testIsIdleNow() throws Exception {
         final int records = 10;
         final int duration = 1000;
@@ -182,11 +182,7 @@ public class DurationFilterTest {
         assertFalse(sf.isIdle());
         assertFalse(sf.isLoggable(r));
 
-        //Cool down and allow.
-        final long then = System.currentTimeMillis();
-        do {
-            Thread.sleep(duration + 100);
-        } while ((System.currentTimeMillis() - then) < duration);
+        tickMilli(duration + 100); //Cool down and allow.
 
         assertTrue(sf.isIdle());
         for (int i = 0; i < records; i++) {
@@ -213,14 +209,14 @@ public class DurationFilterTest {
         for (int i = 0; i < records; i++) {
             ++millis;
             r = new LogRecord(lvl, Long.toString(millis));
-            r.setMillis(millis);
+            setEpochMilli(r, millis);
             assertTrue(Integer.toString(i), sf.isLoggable(r));
         }
 
         //Saturate.
         for (int i = 0; i < records * 10; i++) {
             r = new LogRecord(lvl, Long.toString(millis));
-            r.setMillis(millis);
+            setEpochMilli(r, millis);
             assertFalse(Integer.toString(i), sf.isLoggable(r));
         }
 
@@ -229,7 +225,7 @@ public class DurationFilterTest {
         for (int i = 0; i < records; i++) {
             ++millis;
             r = new LogRecord(lvl, Long.toString(millis));
-            r.setMillis(millis);
+            setEpochMilli(r, millis);
             assertTrue(Integer.toString(i), sf.isLoggable(r));
         }
     }
@@ -250,16 +246,16 @@ public class DurationFilterTest {
         sf.isLoggable(r); //Init the duration.
         millis += (2 * duration) - 1;
         for (int i = 0; i < records - 2; i++) {
-            r.setMillis(millis);
+            setEpochMilli(r, millis);
             assertTrue(Integer.toString(i), sf.isLoggable(r));
         }
 
         millis += 100;
-        r.setMillis(millis);
+        setEpochMilli(r, millis);
         assertTrue(sf.isLoggable(r));
 
         for (int i = 0; i < records - 1; i++) {
-            r.setMillis(millis);
+            setEpochMilli(r, millis);
             assertFalse(Integer.toString(i), sf.isLoggable(r));
         }
     }
@@ -294,7 +290,7 @@ public class DurationFilterTest {
         //Allow
         for (int i = 0; i < records; i++) {
             r = new LogRecord(lvl, Long.toString(millis));
-            r.setMillis(millis);
+            setEpochMilli(r, millis);
             assertTrue(Integer.toString(i), sf.isLoggable(r));
             --millis;
         }
@@ -304,7 +300,7 @@ public class DurationFilterTest {
         final long peak = millis;
         for (int i = 0; i < records; i++) {
             r = new LogRecord(lvl, Long.toString(millis));
-            r.setMillis(millis);
+            setEpochMilli(r, millis);
             assertFalse(Integer.toString(i), sf.isLoggable(r));
         }
 
@@ -312,7 +308,7 @@ public class DurationFilterTest {
         millis = peak + duration;
         for (int i = 0; i < records; i++) {
             r = new LogRecord(lvl, Long.toString(millis));
-            r.setMillis(millis);
+            setEpochMilli(r, millis);
             assertTrue(Integer.toString(i), sf.isLoggable(r));
             ++millis;
         }
@@ -345,13 +341,13 @@ public class DurationFilterTest {
         DurationFilter sf = new DurationFilter(records, records);
         LogRecord r = new LogRecord(Level.INFO, "");
         for (int i = 1; i < (records / 2); i++) {
-            r.setMillis(++millis);
+            setEpochMilli(r, ++millis);
             assertTrue(sf.isLoggable(r));
         }
 
         millis += signum * (60L * 60L * 1000L);
         for (int i = (records / 2); i <= records; i++) {
-            r.setMillis(++millis);
+            setEpochMilli(r, ++millis);
             assertTrue(sf.isLoggable(r));
         }
     }
@@ -363,16 +359,16 @@ public class DurationFilterTest {
         DurationFilter sf = new DurationFilter(records, duration);
         for (int i = 0; i < records; i++) {
             LogRecord r = new LogRecord(Level.INFO, "");
-            r.setMillis(Long.MAX_VALUE);
+            setEpochMilli(r, Long.MAX_VALUE);
             assertTrue(sf.isLoggable(r));
         }
 
         LogRecord r = new LogRecord(Level.INFO, "");
-        r.setMillis(Long.MAX_VALUE);
+        setEpochMilli(r, Long.MAX_VALUE);
         assertFalse(sf.isLoggable(r));
 
         r = new LogRecord(Level.INFO, "");
-        r.setMillis(Long.MAX_VALUE + duration);
+        setEpochMilli(r, Long.MAX_VALUE + duration);
         assertTrue(sf.isLoggable(r));
     }
 
@@ -383,16 +379,16 @@ public class DurationFilterTest {
         DurationFilter sf = new DurationFilter(records, duration);
         for (int i = 0; i < records; i++) {
             LogRecord r = new LogRecord(Level.INFO, "");
-            r.setMillis(Long.MIN_VALUE);
+            setEpochMilli(r, Long.MIN_VALUE);
             assertTrue(Integer.toString(i), sf.isLoggable(r));
         }
 
         LogRecord r = new LogRecord(Level.INFO, "");
-        r.setMillis(Long.MIN_VALUE);
+        setEpochMilli(r, Long.MIN_VALUE);
         assertFalse(sf.isLoggable(r));
 
         r = new LogRecord(Level.INFO, "");
-        r.setMillis(Long.MIN_VALUE + duration);
+        setEpochMilli(r, Long.MIN_VALUE + duration);
         assertTrue(sf.isLoggable(r));
     }
 
@@ -409,7 +405,7 @@ public class DurationFilterTest {
         assertEquals(period, (double) duration / (double) records, 0.0);
         for (int i = 0; i < records * records; i++) {
             r = new LogRecord(lvl, Long.toString(millis));
-            r.setMillis(millis);
+            setEpochMilli(r, millis);
             assertTrue(Integer.toString(i), sf.isLoggable(r));
             millis += period;
         }
@@ -427,7 +423,7 @@ public class DurationFilterTest {
         double period = duration / (double) records;
         for (int i = 0; i < (duration * records) * 2; i++) {
             r = new LogRecord(lvl, Double.toString(millis));
-            r.setMillis((long) millis);
+            setEpochMilli(r, (long) millis);
             assertTrue(Integer.toString(i), sf.isLoggable(r));
             millis = millis + Math.ceil(period);
         }
@@ -444,7 +440,7 @@ public class DurationFilterTest {
         long period = duration / records;
         for (int i = 0; i < records; i++) {
             r = new LogRecord(lvl, Long.toString((long) millis));
-            r.setMillis((long) millis);
+            setEpochMilli(r, (long) millis);
             assertTrue(Integer.toString(i), sf.isLoggable(r));
             millis += period;
         }
@@ -452,14 +448,14 @@ public class DurationFilterTest {
         //Saturated for records + one.
         for (int i = 0; i <= records; i++) {
             r = new LogRecord(lvl, Long.toString((long) millis));
-            r.setMillis((long) millis);
+            setEpochMilli(r, (long) millis);
             assertFalse(Integer.toString(i), sf.isLoggable(r));
             millis += period;
         }
 
         for (int i = 0; i < records; i++) {
             r = new LogRecord(lvl, Long.toString((long) millis));
-            r.setMillis((long) millis);
+            setEpochMilli(r, (long) millis);
             assertTrue(Integer.toString(i), sf.isLoggable(r));
             millis += period;
         }
@@ -471,12 +467,12 @@ public class DurationFilterTest {
         LogRecord r = new LogRecord(lvl, Long.toString(millis));
 
         for (long i = 0; i < records; i++) {
-            r.setMillis(millis);
+            setEpochMilli(r, millis);
             assertTrue(sf.isLoggable(r));
         }
 
         r = new LogRecord(lvl, Long.toString(millis));
-        r.setMillis(millis);
+        setEpochMilli(r, millis);
         assertFalse(sf.isLoggable(r));
     }
 
@@ -571,6 +567,21 @@ public class DurationFilterTest {
         assertTrue(s.contains("loggable="));
     }
 
+    @Test
+    public void testJavaMailLinkage() throws Exception {
+        testJavaMailLinkage(DurationFilter.class);
+    }
+
+    @Test
+    public void testLogManagerModifiers() throws Exception {
+        testLogManagerModifiers(DurationFilter.class);
+    }
+
+    @Test
+    public void testWebappClassLoaderFieldNames() throws Exception {
+        testWebappClassLoaderFieldNames(DurationFilter.class);
+    }
+
     @Test
     public void testInitRecords() throws Exception {
         testInitRecords("210", 210);
@@ -588,7 +599,7 @@ public class DurationFilterTest {
 
     @Test
     public void testInitRecordIso8601() throws Exception {
-        if (isoDurationAllowed()) {
+        if (hasJavaTimeModule()) {
             testInitRecords("PT30M", 1000);
         }
     }
@@ -675,42 +686,42 @@ public class DurationFilterTest {
 
     @Test
     public void testInitDurationIso8601Ms() throws Exception {
-        if (isoDurationAllowed()) {
+        if (hasJavaTimeModule()) {
             testInitDuration("PT0.345S", 345);
         }
     }
 
     @Test
     public void testInitDurationIso8601Sec() throws Exception {
-        if (isoDurationAllowed()) {
+        if (hasJavaTimeModule()) {
             testInitDuration("PT20.345S", (20L * 1000L) + 345);
         }
     }
 
     @Test
     public void testInitDurationIso8601Min() throws Exception {
-        if (isoDurationAllowed()) {
+        if (hasJavaTimeModule()) {
             testInitDuration("PT30M", 30L * 60L * 1000L);
         }
     }
 
     @Test
     public void testInitDurationIso8601Hour() throws Exception {
-        if (isoDurationAllowed()) {
+        if (hasJavaTimeModule()) {
             testInitDuration("PT10H", 10L * 60L * 60L * 1000L);
         }
     }
 
     @Test
     public void testInitDurationIso8601Day() throws Exception {
-        if (isoDurationAllowed()) {
+        if (hasJavaTimeModule()) {
             testInitDuration("P2D", 2L * 24L * 60L * 60L * 1000L);
         }
     }
 
     @Test
     public void testInitDurationIso8601All() throws Exception {
-        if (isoDurationAllowed()) {
+        if (hasJavaTimeModule()) {
             testInitDuration("P2DT3H4M20.345S", (2L * 24L * 60L * 60L * 1000L)
                     + (3L * 60L * 60L * 1000L) + (4L * 60L * 1000L)
                     + ((20L * 1000L) + 345));
@@ -741,22 +752,6 @@ public class DurationFilterTest {
         }
     }
 
-    private boolean isoDurationAllowed() {
-        try {
-            Class.forName("java.time.Duration");
-            return true;
-        } catch (final ClassNotFoundException notSupported) {
-        } catch (final LinkageError notSupported) {
-        }
-        return false;
-    }
-
-    private void read(LogManager manager, Properties props) throws IOException {
-        final ByteArrayOutputStream out = new ByteArrayOutputStream(512);
-        props.store(out, "No comment");
-        manager.readConfiguration(new ByteArrayInputStream(out.toByteArray()));
-    }
-
     public static final class DurationFilterExt extends DurationFilter
             implements Cloneable {
 
