@@ -1,8 +1,8 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 2009-2015 Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 2009-2015 Jason Mehrens. All rights reserved.
+ * Copyright (c) 2009-2016 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009-2016 Jason Mehrens. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
@@ -44,6 +44,7 @@ import java.io.*;
 import java.lang.management.CompilationMXBean;
 import java.lang.management.ManagementFactory;
 import java.lang.reflect.*;
+import java.util.Arrays;
 import java.util.Comparator;
 import java.util.Locale;
 import java.util.Map;
@@ -60,7 +61,7 @@ import static org.junit.Assert.*;
  *
  * @author Jason Mehrens
  */
-public class LogManagerPropertiesTest {
+public class LogManagerPropertiesTest extends AbstractLogging {
 
     /**
      * Holder used to inject Throwables into other APIs.
@@ -94,6 +95,12 @@ public class LogManagerPropertiesTest {
         fullFence();
     }
 
+    @Test
+    public void testDeclaredClasses() throws Exception {
+        Class<?>[] declared = LogManagerProperties.class.getDeclaredClasses();
+        assertEquals(Arrays.toString(declared), 0, declared.length);
+    }
+
     @Test
     public void testCheckAccessPresent() {
         LogManager m = LogManager.getLogManager();
@@ -268,47 +275,12 @@ public class LogManagerPropertiesTest {
 
     @Test
     public void testJavaMailLinkage() throws Exception {
-        for (Method m : LogManagerProperties.class.getDeclaredMethods()) {
-            assertFalse(m.getReturnType().getName(),
-                    isFromJavaMail(m.getReturnType()));
-            for (Class<?> p : m.getParameterTypes()) {
-                assertFalse(p.getName(), isFromJavaMail(p));
-            }
-
-            for (Class<?> e : m.getExceptionTypes()) {
-                assertFalse(e.getName(), isFromJavaMail(e));
-            }
-        }
-
-        for (Constructor<?> c : LogManagerProperties.class.getDeclaredConstructors()) {
-            for (Class<?> p : c.getParameterTypes()) {
-                assertFalse(p.getName(), isFromJavaMail(p));
-            }
-
-            for (Class<?> e : c.getExceptionTypes()) {
-                assertFalse(e.getName(), isFromJavaMail(e));
-            }
-        }
-
-        for (Field f : LogManagerProperties.class.getDeclaredFields()) {
-            assertFalse(f.getName(), isFromJavaMail(f.getType()));
-        }
+        testJavaMailLinkage(LogManagerProperties.class);
     }
 
-    private boolean isFromJavaMail(Class<?> k) throws Exception {
-        for (Class<?> t = k; t != null; t = t.getSuperclass()) {
-            final String n = t.getName();
-            if (n.startsWith("javax.mail.")) {
-                return true;
-            }
-
-            //Not included with logging-mailhandler.jar.
-            if (n.startsWith("com.sun.mail.")
-                    && !n.startsWith("com.sun.mail.util.logging.")) {
-                return true;
-            }
-        }
-        return false;
+    @Test
+    public void testWebappClassLoaderFieldNames() throws Exception {
+        testWebappClassLoaderFieldNames(LogManagerProperties.class);
     }
 
     @Test
@@ -462,14 +434,16 @@ public class LogManagerPropertiesTest {
         }
         Assert.assertFalse(fail);
     }
-    
+
     @Test
     public void testParseDurationMs() throws Exception {
         try {
             long ms = LogManagerProperties.parseDurationToMillis("PT0.345S");
             assertEquals(345L, ms);
         } catch (ClassNotFoundException ignore) {
+            assertFalse(ignore.toString(), hasJavaTimeModule());
         } catch (NoClassDefFoundError ignore) {
+            assertFalse(ignore.toString(), hasJavaTimeModule());
         }
     }
 
@@ -479,7 +453,9 @@ public class LogManagerPropertiesTest {
             long ms = LogManagerProperties.parseDurationToMillis("PT20.345S");
             assertEquals((20L * 1000L) + 345L, ms);
         } catch (ClassNotFoundException ignore) {
+            assertFalse(ignore.toString(), hasJavaTimeModule());
         } catch (NoClassDefFoundError ignore) {
+            assertFalse(ignore.toString(), hasJavaTimeModule());
         }
     }
 
@@ -489,7 +465,9 @@ public class LogManagerPropertiesTest {
             long ms = LogManagerProperties.parseDurationToMillis("PT15M");
             assertEquals(15L * 60L * 1000L, ms);
         } catch (ClassNotFoundException ignore) {
+            assertFalse(ignore.toString(), hasJavaTimeModule());
         } catch (NoClassDefFoundError ignore) {
+            assertFalse(ignore.toString(), hasJavaTimeModule());
         }
     }
 
@@ -499,7 +477,9 @@ public class LogManagerPropertiesTest {
             long ms = LogManagerProperties.parseDurationToMillis("PT10H");
             assertEquals(10L * 60L * 60L * 1000L, ms);
         } catch (ClassNotFoundException ignore) {
+            assertFalse(ignore.toString(), hasJavaTimeModule());
         } catch (NoClassDefFoundError ignore) {
+            assertFalse(ignore.toString(), hasJavaTimeModule());
         }
     }
 
@@ -509,7 +489,9 @@ public class LogManagerPropertiesTest {
             long ms = LogManagerProperties.parseDurationToMillis("P2D");
             assertEquals(2L * 24L * 60L * 60L * 1000L, ms);
         } catch (ClassNotFoundException ignore) {
+            assertFalse(ignore.toString(), hasJavaTimeModule());
         } catch (NoClassDefFoundError ignore) {
+            assertFalse(ignore.toString(), hasJavaTimeModule());
         }
     }
 
@@ -522,7 +504,9 @@ public class LogManagerPropertiesTest {
                     + (3L * 60L * 60L * 1000L) + (4L * 60L * 1000L)
                     + ((20L * 1000L) + 345), ms);
         } catch (ClassNotFoundException ignore) {
+            assertFalse(ignore.toString(), hasJavaTimeModule());
         } catch (NoClassDefFoundError ignore) {
+            assertFalse(ignore.toString(), hasJavaTimeModule());
         }
     }
 
@@ -1175,6 +1159,41 @@ public class LogManagerPropertiesTest {
         }
     }
 
+    @Test
+    @SuppressWarnings("unchecked")
+    public void testGetZonedDateTime() throws Exception {
+        LogRecord r1 = new LogRecord(Level.SEVERE, "");
+        LogRecord r2 = new LogRecord(Level.SEVERE, "");
+        try {
+            final Class<?> k = Class.forName("java.time.ZonedDateTime");
+            setEpochSecond(r1, 100, 1);
+            setEpochSecond(r2, 100, 1);
+            Comparable<Object> c1 = (Comparable<Object>)
+                LogManagerProperties.getZonedDateTime(r1);
+            Comparable<Object> c2 = (Comparable<Object>)
+                LogManagerProperties.getZonedDateTime(r2);
+
+            assertEquals(k, c1.getClass());
+            assertEquals(k, c2.getClass());
+            assertNotSame(c1, c2);
+            assertEquals(c1.getClass(), c2.getClass());
+            assertEquals(0, c1.compareTo(c2));
+        } catch (final NoSuchMethodException preJdk9) {
+            assertNull(LogManagerProperties.getZonedDateTime(r1));
+            assertNull(LogManagerProperties.getZonedDateTime(r2));
+            assertTrue(hasJavaTimeModule());
+        } catch (final ClassNotFoundException preJdk8) {
+            assertNull(LogManagerProperties.getZonedDateTime(r1));
+            assertNull(LogManagerProperties.getZonedDateTime(r2));
+            assertFalse(hasJavaTimeModule());
+        }
+    }
+
+    @Test(expected=NullPointerException.class)
+    public void testGetZonedDateTimeNull() throws Exception {
+        LogManagerProperties.getZonedDateTime((LogRecord) null);
+    }
+
     private static void setPending(final Throwable t) {
         if (t != null) {
             PENDING.set(t);
@@ -1218,12 +1237,6 @@ public class LogManagerPropertiesTest {
         throw new AssertionError();
     }
 
-    private void read(LogManager manager, Properties props) throws IOException {
-        final ByteArrayOutputStream out = new ByteArrayOutputStream(512);
-        props.store(out, "No comment");
-        manager.readConfiguration(new ByteArrayInputStream(out.toByteArray()));
-    }
-
     private boolean contains(Properties props, String key, String value) {
         if (key == null) {
             throw new NullPointerException();
@@ -1267,6 +1280,7 @@ public class LogManagerPropertiesTest {
             throwPendingIfSet();
         }
 
+        @SuppressWarnings("override") //JDK-6954234
         public int compare(LogRecord r1, LogRecord r2) {
             throw new Error("");
         }
@@ -1277,10 +1291,12 @@ public class LogManagerPropertiesTest {
 
         private static final long serialVersionUID = 1L;
 
+        @SuppressWarnings("override") //JDK-6954234
         public int compare(LogRecord r1, LogRecord r2) {
             throw new UnsupportedOperationException();
         }
 
+        @SuppressWarnings("override")
         public Comparator<LogRecord> reversed() {
             return new DescComparator();
         }
@@ -1291,10 +1307,12 @@ public class LogManagerPropertiesTest {
 
         private static final long serialVersionUID = 1L;
 
+        @SuppressWarnings("override") //JDK-6954234
         public int compare(LogRecord r1, LogRecord r2) {
             throw new UnsupportedOperationException();
         }
 
+        @SuppressWarnings("override")
         public Comparator<LogRecord> reversed() {
             return new AscComparator();
         }
@@ -1306,6 +1324,7 @@ public class LogManagerPropertiesTest {
             throwPendingIfSet();
         }
 
+        @SuppressWarnings("override") //JDK-6954234
         public boolean isLoggable(LogRecord record) {
             throw new Error("");
         }
@@ -1317,6 +1336,7 @@ public class LogManagerPropertiesTest {
             throwPendingIfSet();
         }
 
+        @Override
         public String format(LogRecord record) {
             throw new Error("");
         }
