@@ -1,8 +1,8 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 2013-2014 Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 2013-2014 Jason Mehrens. All rights reserved.
+ * Copyright (c) 2013-2016 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013-2016 Jason Mehrens. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
@@ -42,7 +42,6 @@ package com.sun.mail.util.logging;
 
 import java.io.*;
 import java.lang.reflect.InvocationTargetException;
-import java.lang.reflect.Modifier;
 import java.lang.reflect.UndeclaredThrowableException;
 import java.nio.channels.ClosedByInterruptException;
 import java.nio.channels.FileLockInterruptionException;
@@ -58,7 +57,13 @@ import org.junit.Test;
  * @author Jason Mehrens
  * @since JavaMail 1.5.2
  */
-public class SeverityComparatorTest {
+public class SeverityComparatorTest extends AbstractLogging {
+
+    @Test
+    public void testDeclaredClasses() throws Exception {
+        Class<?>[] declared = SeverityComparator.class.getDeclaredClasses();
+        assertEquals(Arrays.toString(declared), 0, declared.length);
+    }
 
     @Test
     public void testApplyNull() {
@@ -584,7 +589,8 @@ public class SeverityComparatorTest {
         LogRecord r1 = new LogRecord(Level.INFO, Level.INFO.toString());
         LogRecord r2 = new LogRecord(Level.INFO, Level.INFO.toString());
         r1.setSequenceNumber(r2.getSequenceNumber());
-        r1.setMillis(r2.getMillis());
+        setEpochMilli(r2, System.currentTimeMillis()); //Truncate nanos.
+        setEpochMilli(r1, r2.getMillis());
 
         assertEquals(r1.getLevel(), r2.getLevel());
         assertEquals(r1.getSequenceNumber(), r2.getSequenceNumber());
@@ -1129,12 +1135,19 @@ public class SeverityComparatorTest {
         assertEquals(a.toString(), a.toString());
     }
 
+    @Test
+    public void testJavaMailLinkage() throws Exception {
+        testJavaMailLinkage(SeverityComparator.class);
+    }
+
     @Test
     public void testLogManagerModifiers() throws Exception {
-        final Class<?> c = SeverityComparator.class;
-        assertTrue(Modifier.isPublic(c.getModifiers()));
-        assertFalse(Modifier.isStatic(c.getModifiers()));
-        assertTrue(Modifier.isPublic(c.getConstructor().getModifiers()));
+        testLogManagerModifiers(SeverityComparator.class);
+    }
+
+    @Test
+    public void testWebappClassLoaderFieldNames() throws Exception {
+        testWebappClassLoaderFieldNames(SeverityComparator.class);
     }
 
     @Test
@@ -1185,8 +1198,8 @@ public class SeverityComparatorTest {
         LogRecord r1 = new LogRecord(Level.INFO, Level.INFO.toString());
         LogRecord r2 = new LogRecord(Level.INFO, Level.INFO.toString());
         r2.setSequenceNumber(r1.getSequenceNumber());
-        r1.setMillis(10);
-        r2.setMillis(20);
+        setEpochMilli(r1, 10);
+        setEpochMilli(r2, 20);
 
         assertEquals(r1.getLevel(), r2.getLevel());
         assertEquals(r1.getThrown(), r2.getThrown());
